"""
Module with vulnerability checks.
"""

from vulnerability_checker.checks.base import BaseCheck
from vulnerability_checker.checks.sql_injection import SQLInjectionCheck
from vulnerability_checker.checks.xss import XSSCheck
from vulnerability_checker.checks.dependency import DependencyCheck
from vulnerability_checker.checks.insecure_deserialization import InsecureDeserializationCheck
from vulnerability_checker.checks.secret_leak import SecretLeakCheck
from vulnerability_checker.checks.code_injection import CodeInjectionCheck

# Registration of checks
AVAILABLE_CHECKS = {
    "sql-injection": SQLInjectionCheck,
    "xss": XSSCheck, 
    "dependency-check": DependencyCheck,
    "insecure-deserialization": InsecureDeserializationCheck,
    "secret-leak": SecretLeakCheck,
    "code-injection": CodeInjectionCheck,
} 